#include <p18f27j13.h>
#include <timers.h>
#include <pwm.h>
#include <delays.h>
#include <stdlib.h>
#include <portb.h>
#include "c_what.h"
#include "lj_yeah2.h"
#include "lj_yeah3.h"
#include "lj_yeah4.h"
#include "lj_okay.h"
#include "lj_what.h"
#include "c_yeah1.h"
#include "c_yeah2.h"
#include "c_yeah3.h"
#include "c_yeah4.h"
#include "c_okay.h"

//number of wavetables
const int NSOUND = 11;
int cSound = 0; //current sound selected
int cIdx = -1; //current sample index
unsigned long iTime = 0; //idle time
//array of pointers to wavetables
rom char* wPtr[] = {c_what, lj_yeah2, lj_yeah3, lj_yeah4, lj_okay, lj_what, c_yeah1, c_yeah2, c_yeah3, c_yeah4, c_okay};
//array of wavetable lengths
int wSizes[] = {sizeof(c_what), sizeof(lj_yeah2), sizeof(lj_yeah3), sizeof(lj_yeah4), sizeof(lj_okay), sizeof(lj_what), sizeof(c_yeah1), sizeof(c_yeah2), sizeof(c_yeah3), sizeof(c_yeah4), sizeof(c_okay)};
int left_sample, right_sample;
int cSubIdx = 0;

//ISR for TIMER/PWM and INT0
#pragma code
#pragma interrupt InterruptHandlerHigh
void InterruptHandlerHigh ()
{
	//check for TMR2 (PWM needs new data)
	if (PIR1bits.TMR2IF)
	{	
		PIR1bits.TMR2IF = 0;			//clear interrupt flag
		if (cIdx >= 0)
		{
			//Load data into PWM Duty Cycle Register
			int current_sample = wPtr[cSound][cIdx];
			CCPR4L = current_sample;
				
			cIdx ++;
			 
			if (cIdx > wSizes[cSound])
			{
				//map PWM to default (something else, simpler than stopping)
				T2CONbits.TMR2ON = 0; // Disable Timer/PWM
				cIdx = -1;	//setting it to negative value means not playing
			}
			iTime = 0;
		}		
		
	}
	if (INTCONbits.INT0IF)
	{
		INTCONbits.INT0IF = 0;
		//switch debounce, 1/8 second
		if (cIdx > 0 && cIdx < 1500)
			return;
		//play a new sound if previously stopped or past 3/8th of a second
		//else randomly pick a sound
		if (cIdx < 0 || cIdx > 3000)
			cSound = rand()%NSOUND;
		cIdx = 0;
		// Enable Timer/PWM
		CCP4CONbits.CCP4M = 0b1100;
		T2CONbits.TMR2ON = 1; 
		iTime = 0;
	}
}


//Interrupt vector, jump to ISR
#pragma code InterruptVectorHigh = 0x08
void InterruptVectorHigh (void)
{
	_asm
		goto InterruptHandlerHigh //jump to interrupt routine
	_endasm
}

void setupINT()
{
	//Enable interrupt INT0, this is the input for the pin
	INTCONbits.INT0IF = 0;
	INTCONbits.INT0IE = 1;
}

void setupPWM()
{
	
	//Initialize and enable CCP4 (PWM)
	CCPTMRS1bits.C4TSEL = 0b00; // Select Timer 2 as output compare time base
	
	CCP4CONbits.DC4B = 0b11;
	CCP4CONbits.CCP4M = 0b1100;
	CCPR4L = 0x0F;
		
	//Initialize and enable Timer2
	T2CONbits.TMR2ON = 0; // Disable Timer
	T2CONbits.T2CKPS = 0b00; // Select 1:1 Prescaler
	T2CONbits.T2OUTPS = 0b0011; // Select 1:4 Postscaler (update PWM every 4 PWM cycles)
	TMR2 = 0x0; // Clear timer register 
	PR2 = 0xFF; // Load the period value (For 32KHz PWM at 48MHz Fosc)
	PIR1bits.TMR2IF = 0; // Clear Timer 2 Interrupt Flag
	PIE1bits.TMR2IE = 1; // Enable Timer 2 interrupt
	T2CONbits.TMR2ON = 1; // Start Timer
}

void setupIO()
{
	//Sets up the pin map for peripherals
	//Disables all analog inputs
	ANCON0 = 0xFF;
	ANCON1 = 0xFF;
	//Set all unconnected pins to digital output, low, to save power
	TRISA = 0;
	TRISB = 0;
	TRISC = 0;
	PORTA = 0;
	PORTB = 0;
	PORTC = 0;
	//Set switch port to input and enable internal pullup
	PORTBbits.RB0 = 1;
	TRISBbits.TRISB0 = 1;
	INTCON2bits.RBPU = 0;
	//Set PWM port to output
	PORTBbits.RB4 = 0;
	TRISBbits.TRISB4 = 0;
	//Set (optional) LED debug pin to output
	PORTBbits.RB3 = 0;
	TRISBbits.TRISB3 = 0;
}

void main ()
{

	setupIO();
	setupPWM();
	setupINT();
	
	//Enable interrupts
	RCONbits.IPEN = 1;
	INTCONbits.GIEH	= 1;

	while (1)
	{
	
		if (iTime > 0x000FFFFF)
		{
			T2CONbits.TMR2ON = 0; // Disable Timer/PWM
			CCP4CONbits.CCP4M = 0b0000;
			PORTBbits.RB4 = 0;
			PORTBbits.RB3 = 0;
			iTime = 0;
			Sleep();
		}
		else
		{
			PORTBbits.RB3 = 1;
			iTime ++;
		}
	}	
}
